﻿package plugins
{
	import flash.display.*;
	import flash.events.*;
	import flash.net.*;
	import flash.media.*;
	import flash.geom.*;
	import flash.filesystem.*;
	import com.adobe.images.PNGEncoder;
	import agf.utils.FileUtils;
	import agf.utils.FileInfo;
	import flash.utils.ByteArray;
	
	public class WaveFormEx
	{
		public function WaveFormEx (args:Array=null) {
			_args = args;
		}
		
		public static var superType:String="audio"; // InputTextBox-type inheritance: empty string or number, string, screennumber, file, image etc..
		
		private var _container:Object /*InputTextBox*/;
		private var _args:Array;
		private var _pageItemName:String = "";
		private var _text:String = "";
		private var _width:int=0;
		private var _height:int = 124;
		
		private var channel:SoundChannel;
		private var sound:Sound;
		private var samples:ByteArray;
		private var buffer:BitmapData;
		private var screen:Bitmap;
		private var screenDisplay:Bitmap;
		
		private var rect:Rectangle = new Rectangle(0,0,1,0);
		private var playingTime:int;
		private var step:int;
		
		private var _audioPath:String;
		private var _app:Object;
		
		private static var waveColor:uint = 0xFFFFFFFF;

		private static var BMP_WIDTH:int = 1024;
		private static var BMP_HEIGHT:int = 512;
		private static var BMP_BGCOLOR:uint = 0x00000000;

		private static var pageItems:Object = {};

		public static function getName () :String {
			return "WaveFormEx";
		}

		// Audio Input with waveform bitmap generator
		// {#file:Plugin('Wave','plugins.WaveFormEx','mp3','audio-#inputname#.#extension#','Mp3 Files','*.mp3;')} -> returns the mp3 file
		//
		// Member bmp:
		// {#mywave@bmp} -> returns path to wave form bitmap (mp3/file.png)
		// {#mywave@width} -> returns width of wave form bitmap (1024)
		// {#mywave@height} -> returns height of wave form bitmap (512)
		public static function getMember ( pageItemName:String, memberName:String ) :String
		{
			if( pageItems && pageItems[pageItemName] ) {
				try {
					return pageItems[pageItemName][memberName];
				}catch(e:Error) {
					trace("Error Member: "+ memberName + " not found in " + pageItemName );
				}
			}
			
			return "";
		}
		
		// Create extension ui (inside input-text-box)
		public function init ( c:Object /*InputTextBox*/, app:Object /*Application*/, pageItemName:String, value:String ) :void {
			if( c ) {
				_app = app;
				_container = c;
				_pageItemName = pageItemName;

				var t:Object = _app.getClass("ct.CTTools");
				var fi:FileInfo = FileUtils.fileInfo( value );
				var bmpfile:String = t.projectDir + fi.separator + "raw" + fi.separator + fi.directory + fi.separator + fi.name + ".png";
				
				if( t.fileExists(bmpfile) )
				{
					if( screen && _container.contains( screen) ) _container.removeChild(screen);

					pageItems[ _pageItemName ] = { 
						bmp: fi.directory + fi.separator + fi.name + ".png",
						width: 1024,
						height: 512
					};
					
					//_container.loadImage( bmpfile );
					// file:String, fin:Function=null, isTextFile:Boolean=true, binaryTextFile:Boolean = false, clearFileCache:Boolean=false,
					_app.instance.resourceMgr.loadResource( bmpfile, onCached, false, false, true );
				}
				
				setWidth( _container.getWidth() );
			}
		}
		
		// Returns the output html text
		public function getText () :String {
			return _text;
		}
		
		// Align extension to parent container (on stage resize)
		public function setWidth (w:int) :void
		{
			_width = w;
			
			var sp:DisplayObject;
			
			if( screen ) sp = DisplayObject(screen);
			else if( screenDisplay ) sp = DisplayObject(screenDisplay);
			
			if( sp )
			{
				_height = BMP_HEIGHT/(BMP_WIDTH/w) + _container.textField.textHeight + _container.textField.y;

				sp.y = _container.textField.textHeight + _container.textField.y;
				sp.width = w;
				sp.height = _height;

				// Set height of input container
				_container.setHeight( _height );
				
				var itp:Class = _app.getClass("ct.ctrl.InputTextBox");
				if( itp ) {
					itp.heightDirty = true;
				}
			}
		}
		
		// Extension controls the height
		public function getHeight () :int {
			return _height;
		}
		
		// Set input text, called by the app after init() and textField user change events
		public function setText ( text:String ) :void
		{
			_text = text;
			
			trace("WaveForm::setText: " + text );
			
			if( text == "" || text == "none" ) {
				if( screen && _container.contains( screen) ) _container.removeChild(screen);
				if( screenDisplay && _container.contains( screenDisplay) ) _container.removeChild(screenDisplay);
				return;
			}
			
			// Get access to ct.CTTools
			var t:Object = _app.getClass("ct.CTTools");

			var fi:FileInfo = FileUtils.fileInfo( text );
			var bmpfile:String = t.projectDir + fi.separator + "raw" + fi.separator + fi.directory + fi.separator + fi.name + ".png";
			
			// generate the wave-bmp from the selected mp3 file
			writeBmp( t.projectDir + fi.separator + "raw" + fi.separator + _text );
		
		}
		

		private function onCached( r:Object ) :void {
			if( r.loaded == 1 ) {
				
				var d:DisplayObject = DisplayObject( r.obj );
				
				if( d )
				{
					var tmpW:int = d.width;
					var tmpH:int = d.height;
		
					var sclX:Number = _width / tmpW;
					
					if( sclX < 1 ) {
						d.width *= sclX;
						d.height *= sclX;
					}
					
					screenDisplay = Bitmap( d );
					
					_container.addChild( d );
					
					//d.y = 24;
					
					setWidth( _width );
					
					_container.dispatchEvent( new Event("heightChange") );
				}
			}
		}
		
		private function writeBmp ( path:String ) :void
		{
			_audioPath = path;
			
			if( screen && _container.contains( screen) ) _container.removeChild(screen);
			
			samples = new ByteArray();
			buffer = new BitmapData(BMP_WIDTH,BMP_HEIGHT,true,BMP_BGCOLOR);
			
			screen = new Bitmap(buffer);
			screen.name = "screen";
			screen.width = _width;
			screen.height = _height;
			sound = new Sound();
			
			if( path )
			{
				Sprite(_container).addChild( screen );
				sound.addEventListener(Event.COMPLETE, loadSoundComplete);
				sound.load( new URLRequest(path) );
			}
		}
		
		private function loadSoundComplete (event:Event) :void
		{
			buffer.fillRect( buffer.rect, 0 );
			
			playingTime = sound.length;
			var extract:Number = Math.floor ((playingTime/1000)*44100);
			
			var lng:Number = sound.extract(samples,extract);
			samples.position = 0;

			step = samples.length/4096;

			do step-- while ( step % 4 );
			
			var left:Number;
			var right:Number;

			var bmp_h2:int = BMP_HEIGHT * 0.5;
			var rv:uint = 0xFFFFFFFF;
			var scl:Number = bmp_h2 * 0.5;
			var tmp:Number;

			for (var c:int = 0; c < 4096; c++)
			{
				rect.x = int(c/4);
				
				left = Math.abs(samples.readFloat()) * scl;
				right = Math.abs(samples.readFloat()) * scl;
				
				samples.position = c*step;

				rect.y = bmp_h2 - left;
				rect.height = left + right;
				
				buffer.fillRect( rect, rv );
			}
			
			setWidth( _width );
			
			var b:ByteArray = PNGEncoder.encode( buffer );
			var fi:FileInfo = FileUtils.fileInfo( _audioPath );
			var nam:String =  fi.directory + fi.separator + fi.name + ".png";
			
			// store member var 
			pageItems[ _pageItemName ] = { 
				bmp: nam,
				width: BMP_WIDTH,
				height: BMP_HEIGHT
			};
			
			var t:Object = _app.getClass("ct.CTTools");

			// write wave-bmp into /raw and /min directory
			t.writeBinaryFile ( t.projectDir + fi.separator + "min" + fi.separator + nam, b );
			t.writeBinaryFile ( t.projectDir + fi.separator + "raw" + fi.separator + nam, b );

			_app.getClass("agf.tools.Console").log("Waveform bitmap written to " + nam );
		}
		
	}
	
}
